/*
 * inetd/signals.c --
 *
 *      This module implements all signal related functions. In
 *      particular, SIGUSR1 signals are handled by writing status
 *      information to stderr.
 */

#define _POSIX_C_SOURCE 2
#define _DEFAULT_SOURCE
#define _DARWIN_C_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <syslog.h>

#include "inetd.h"

#define UNUSED(x)	(void)(x)

/*
 * The SIGUSR1 signal won't be delivered when executing this function,
 * so it is safe to print to the buffered stream.
 */

static void 
handle_sigusr1(int sig)
{
    inetd_service_t *sp;
    inetd_client_t *cp;
    int i;

    UNUSED(sig);

    fprintf(stderr, "Services configured on inetd:\n");
    for (sp = inetd_services; sp; sp = sp->next) {
	fprintf(stderr, "%s\t%s\t", sp->port, sp->name);
	for (i = 0; sp->params[i]; i++) {
	    fprintf(stderr, "%s%s", i ? " " : "", sp->params[i]);
	}
	fprintf(stderr, "\n");
    }
    fprintf(stderr, "\n");

    fprintf(stderr, "Clients connected to inetd:\n");
    for (cp = inetd_clients; cp; cp = cp->next) {
	fprintf(stderr, "%s%s%s:%s\t%s%s%s:%s\t%d\t%s\n",
		strchr(cp->local_host, ':') == NULL ? "" : "[",
		cp->local_host,
		strchr(cp->local_host, ':') == NULL ? "" : "]",
		cp->local_serv,
		strchr(cp->peer_host, ':') == NULL ? "" : "[",
		cp->peer_host,
		strchr(cp->peer_host, ':') == NULL ? "" : "]",
		cp->peer_serv,
		cp->pid, cp->service->name);
    }

    fflush(stderr);
}

/*
 * Install a signal handler for SIGUSR1. Make sure SIGUSR1 is not
 * delivered while the signal handler is active.
 */

void
inetd_signals()
{
    struct sigaction sa;
    sa.sa_handler = handle_sigusr1;
    sa.sa_flags = 0;
    sigemptyset(&sa.sa_mask);
    sigaddset(&sa.sa_mask, SIGUSR1);
    if (sigaction(SIGUSR1, &sa, NULL) == -1) {
	syslog(LOG_ERR, "sigaction() failed: %m - exiting");
	exit(EXIT_FAILURE);
    }
}

